local ADDON_NAME, namespace = ...
local L = namespace.L

--+ types:
--+---+-----------------------
--+ a + achievement
--+ t + toy
--+ m + transMog
--+ p + pet
--+ n + mouNt
--+ i + item
--+ h + headline
--+---+-------------------【ツ】

local PlayerGUID = UnitGUID("player")

local LOOT_ITEM_CREATED_PATTERN
local LOOT_ITEM_CREATED_MULTIPLE_PATTERN 
local LOOT_ITEM_PATTERN
local LOOT_ITEM_MULTIPLE_PATTERN
local LOOT_ITEM_PUSHED_PATTERN
local LOOT_ITEM_PUSHED_MULTIPLE_PATTERN

Recent = LibStub("AceAddon-3.0"):NewAddon("Recent", "AceConsole-3.0", "AceEvent-3.0")

RecentLDB = LibStub("LibDataBroker-1.1"):NewDataObject("Recent", {
    type = "data",
    text = "Recent",
    icon = "Interface\\Icons\\INV_Inscription_Trinket02.blp",
	OnClick = function() if RecentListFrame:IsShown() then RecentListFrame:Hide() else RecentListFrame:Show() end end,
	OnTooltipShow = function(tooltip)
		tooltip:AddLine("Recent")
		tooltip:AddLine(L["|cFFff4040Click|r to open log"], 0.2, 1, 0.2)
	end,
})
local icon = LibStub("LibDBIcon-1.0")

local defaults = {
		profile = {
			minimap = {
				hide = false,
			},
		},
		char = {
			history = {}
		} 
	}

function Recent:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RecentPrefsDB", defaults)
	icon:Register("Recent", RecentLDB, self.db.profile.minimap)

	self:RegisterChatCommand("rct", "OnChatCommand")
	self:RegisterChatCommand("recent", "OnChatCommand")

	RecentDB = self.db.char.history

	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("ACHIEVEMENT_EARNED")
	self:RegisterEvent("CRITERIA_EARNED")
	self:RegisterEvent("LOOT_ITEM_ROLL_WON")
	self:RegisterEvent("TRANSMOG_COLLECTION_SOURCE_ADDED")
--	self:RegisterEvent("GET_ITEM_INFO_RECEIVED")
	self:RegisterEvent("NEW_PET_ADDED")

	RecentListFrame.ListScrollFrame.update = RecentUpdateList;
	RecentListFrame.RecentJunkButton:SetScript("OnClick", RecentJunk_OnClick)
	
	LOOT_ITEM_BONUS_ROLL_MULTIPLE_PATTERN =LOOT_ITEM_BONUS_ROLL_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
	LOOT_ITEM_BONUS_ROLL_PATTERN = LOOT_ITEM_BONUS_ROLL_SELF:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_CREATED_PATTERN = LOOT_ITEM_CREATED_SELF:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_CREATED_MULTIPLE_PATTERN = LOOT_ITEM_CREATED_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
	LOOT_ITEM_PATTERN = LOOT_ITEM_SELF:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_MULTIPLE_PATTERN = LOOT_ITEM_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
	LOOT_ITEM_PUSHED_PATTERN = LOOT_ITEM_PUSHED_SELF:gsub("%%s", "(.+)"):gsub("^", "^")
	LOOT_ITEM_PUSHED_MULTIPLE_PATTERN = LOOT_ITEM_PUSHED_SELF_MULTIPLE:gsub("%%s", "(.+)"):gsub("%%d", "(%%d+)"):gsub("^", "^")
		
	local temp=""
	local ctr=0
	local threshold=7                                    --   DAYS 2 KEEP

	for v, p in pairs(RecentDB) do
	   if p.sd ~= temp then
		  temp=p.sd
		  ctr=ctr+1
	   end   
	end   

	temp=""												-- remove older entries
	while ctr>threshold do
	   
	   if temp == "" then temp=RecentDB[1].sd end 
	   if temp == RecentDB[1].sd then
			table.remove(RecentDB,1)
	   else     
		  ctr=ctr-1
		  temp=""
	   end
	end

	getglobal(RecentListFrame.RecentJunkButton:GetName() .. "Text"):SetText(L[" show me everything"])
	Recent:RecentRecentNewDay()
	Recent:RecentCreateFilteredList()
	RDB=RecentFiltered
	HybridScrollFrame_CreateButtons(RecentListFrame.ListScrollFrame, "RecentListButtonTemplate", 46, 0)   
	RecentUpdateList()

end

function Recent:CHAT_MSG_LOOT(event, ...)	-----------------------------------------------------

	local message, _, _, _, _, _, _, _, _, _, _, guid = ...

	if guid == PlayerGUID then 
		local link, quantity = message:match(LOOT_ITEM_MULTIPLE_PATTERN)
		if not link then
			link, quantity = message:match(LOOT_ITEM_PUSHED_MULTIPLE_PATTERN)
			if not link then
				link, quantity = message:match(LOOT_ITEM_CREATED_MULTIPLE_PATTERN)
				if not link then
					link, quantity = message:match(LOOT_ITEM_BONUS_ROLL_MULTIPLE_PATTERN)
					if not link then
						quantity, link = 1, message:match(LOOT_ITEM_PATTERN)
						if not link then
							quantity, link = 1, message:match(LOOT_ITEM_PUSHED_PATTERN)
							if not link then
								quantity, link = 1, message:match(LOOT_ITEM_CREATED_PATTERN)
								if not link then
									quantity, link = 1, message:match(LOOT_ITEM_BONUS_ROLL_PATTERN)
								end
							end
						end
					end		
				end
			end
		end

		if link then
			local itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture,
				itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(link)

			local a, color, b, linkType, linkID = strsplit(":|H", link)				
			local itemID, toyName, icon, isFavorite, hasFanfare = C_ToyBox.GetToyInfo(linkID)
			local itype= "i"

			if (itemID ~= nil)  then itype="t" 
			elseif (itemClassID==LE_ITEM_CLASS_MISCELLANEOUS and itemSubClassID==LE_ITEM_MISCELLANEOUS_COMPANION_PET) then itype="p"
			elseif (itemClassID==LE_ITEM_CLASS_MISCELLANEOUS and itemSubClassID==LE_ITEM_MISCELLANEOUS_MOUNT) then itype="n"
			end

			local timestamp = time()

			if Recent:sameAsBefore(link) then
				local max = table.getn(RecentDB)
				local cnt
				if RecentDB[max].qnty == nil then cnt =1 else cnt=RecentDB[max].qnty end
				RecentDB[max].qnty = cnt + quantity
				RecentUpdateList()    
			else
				if itemRarity~= nil then
					if itemRarity > 2 or itype=="p" or itype=="n" or itype=="t" then 
						Recent:RecentRecentNewDay()
						table.insert(RecentDB, {ts=timestamp , lnk=link ,qnty= quantity , sd=Recent:sortDate(time()), loc= (string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=itemID, type=itype, addinfo=nil })  
						Recent:RecentCreateFilteredList()
						RecentUpdateList()  
					end   
				end
			end
		end
	end
end

function Recent:ACHIEVEMENT_EARNED(eventName, ...)  -----------------------------------------
	local achID, already = ...
	if achID then 
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuild, wasEarnedByMe, earnedBy = GetAchievementInfo(achID)

		local link = GetAchievementLink(achID)
		Recent:RecentRecentNewDay()
		local timestamp = time()
		table.insert(RecentDB, {ts=timestamp , lnk=link ,qnty=nil , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=achID, type="a", addinfo=nil})  
		Recent:RecentCreateFilteredList()
		RecentUpdateList()
	end
end

function Recent:CRITERIA_EARNED(eventName, ...) -------------------------------------------
	local achID, criteriaString  = ...
	if achID then
		local id, name, points, completed, month, day, year, description, flags, icon, rewardText, isGuild, wasEarnedByMe, earnedBy = GetAchievementInfo(achID)

		local link = GetAchievementLink(achID)
		Recent:RecentRecentNewDay()
		local timestamp = time()
		table.insert(RecentDB, {ts=timestamp , lnk=link ,qnty=nil , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=achID, type="a", addinfo=criteriaString})  
		Recent:RecentCreateFilteredList()
		RecentUpdateList()  
	end 
end

function Recent:LOOT_ITEM_ROLL_WON(eventName, ...)  ----------------------------------------

	local link, quantity, rollType, roll, isUpgraded = ...
	local itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType,
		itemSubType, itemStackCount, itemEquipLoc, texture,
		itemSellPrice, itemClassID, itemSubClassID, bindType, expacID,
		itemSetID, isCraftingReagent = GetItemInfo(link)

	local a, color, b, linkType, linkID = strsplit(":|H", RecentDB[self:GetParent().index].lnk);				
	local itemID, toyName, icon, isFavorite, hasFanfare = C_ToyBox.GetToyInfo(linkID)

	local itype= "i"
	if itemID then itype="t" 
	elseif (itemClassID==LE_ITEM_CLASS_MISCELLANEOUS and itemSubClassID==LE_ITEM_MISCELLANEOUS_COMPANION_PET) then itype="p"
	elseif (itemClassID==LE_ITEM_CLASS_MISCELLANEOUS and itemSubClassID==LE_ITEM_MISCELLANEOUS_MOUNT) then itype="n"
	end

	local timestamp = time()
	
	if itemRarity > 2 or itype=="p" or itype=="n" or itype=="t" then		
		Recent:RecentRecentNewDay()
		table.insert(RecentDB, {ts=timestamp , lnk=link ,qnty= quantity , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) ,  iid=itemID, type=itype, addinfo=nil})  
		Recent:RecentCreateFilteredList()
		RecentUpdateList() 
	end   
end

function Recent:TRANSMOG_COLLECTION_SOURCE_ADDED(eventName, ...)  ---------------------------------------
	sourceID, isAdded, attempt = ...
	if sourceID then
		local _, visualID, _, icon, _, _, link = C_TransmogCollection.GetAppearanceSourceInfo(sourceID)
		local timestamp = time()
		if string.len(link)>0 then
			Recent:RecentRecentNewDay()
			table.insert(RecentDB, {ts=timestamp , lnk=link ,qnty= nil , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=sourceID, type="m", addinfo=nil}) 
			Recent:RecentCreateFilteredList()
			RecentUpdateList()
		end
	end
end

--function Recent:GET_ITEM_INFO_RECEIVED(eventName, ...)   ------------------------------------------------
--	if RecentListFrame:IsShown() then 
--		RecentUpdateList()
--	end
--	DEFAULT_CHAT_FRAME:AddMessage("|cFFFF5555Recent:|r update erhalten", 1.0, 0.8, 0.2);
--end

function Recent:NEW_PET_ADDED(eventName, ...) ----------------------------------------------------------
	petID = ...
	if petID then
		local link = C_PetJournal.GetBattlePetLink(petID)
		local timestamp = time()
		Recent:RecentRecentNewDay()
		table.insert(RecentDB, {ts=timestamp , lnk=link ,qnty= nil , sd=Recent:sortDate(time()), loc=(string.len(GetSubZoneText())>0 and GetZoneText() .. ": " .. GetSubZoneText() or GetZoneText()) , iid=nil, type="p", addinfo=nil}) 
		Recent:RecentCreateFilteredList()
		RecentUpdateList()
	end
end

function Recent:RecentRecentNewDay()
	local max = table.getn(RecentDB)
	if max>0 and (Recent:sortDate(time()) ~= RecentDB[max].sd and RecentDB[max].type ~= 'h') then
		table.insert(RecentDB, {ts=RecentDB[max].ts , lnk='|cff9d9d9d|Hitem:4784::::::::60:::::|h[Lifeless Stone]|h|r' ,qnty= nil, sd=RecentDB[max].sd, loc=nil , iid=nil, type='h', addinfo=nil}) 
	end
end

function Recent:RecentCreateFilteredList()
	if RecentFiltered and RecentDB then				-- only update latest new item
		local max = table.getn(RecentDB)
		if max>=1 then
			local p = RecentDB[max]
			local itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(p.lnk)
			if p.type ~= "i" or  itemClassID== 2 or (itemClassID== 4 and not (itemEquipLoc == nil or itemEquipLoc == '')) or itemClassID== 9  then
				table.insert(RecentFiltered, {ts=p.ts , lnk=p.lnk ,qnty= p.qnty , sd=p.sd, loc=p.loc , iid=p.iid, type=p.type, addinfo=p.addinfo})
			end
		end
	else
	  RecentFiltered={}							-- initialize FilterList
	  local again=0
	  for v, p in pairs(RecentDB) do  
			local itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(p.lnk)
			if p.type ~= "i" or  itemClassID== 2 or (itemClassID== 4 and not (itemEquipLoc == nil or itemEquipLoc == '')) or itemClassID== 9  then
				table.insert(RecentFiltered, {ts=p.ts , lnk=p.lnk ,qnty= p.qnty , sd=p.sd, loc=p.loc , iid=p.iid, type=p.type, addinfo=p.addinfo}) 
				if p.type == 'h' and again== 1 then 
					table.remove(RecentFiltered,table.getn(RecentFiltered)-1)
				elseif p.type == 'h' then
					again=1
				else
					again=0
				end
			end
		end
	end
end

function Recent:GetFullDate(weekday, month, day, year)
	local weekdayName = CALENDAR_WEEKDAY_NAMES[weekday];
	local monthName = CALENDAR_FULLDATE_MONTH_NAMES[month];
	return weekdayName, monthName, day, year, month;   
end

function Recent:sameAsBefore(itemlink)
	local max = table.getn(RecentDB)

	if max>1 then
		if RecentDB[max].type == 'h' then max = max -1 end
			if itemlink==RecentDB[max].lnk then
				local m,d,y = string.match(date("%x",RecentDB[max].ts), '(%d+)/(%d+)/(%d+)')
				local hour = string.match(date("%H", RecentDB[max].ts), '(%d+)')
				local minutes = string.match(date("%M", RecentDB[max].ts), '(%d+)')
				local m1,d1,y1 = string.match(date("%x",time()), '(%d+)/(%d+)/(%d+)')
				local hour1 = string.match(date("%H", time()), '(%d+)')
				local minutes1 = string.match(date("%M", time()), '(%d+)')
				if m==m1 and d==d1 and y==y1 and (minutes==minutes1 or minutes + 1==minutes1) and hour==hour1 then
					return true
				else 
					return false
				end
			else
				return false
			end
	else
		return false
	end
end 

function Recent:localizedDate(...)
	local weekday=date("%w",...)
	local m,d,y = string.match(date("%x",...), '(%d+)/(%d+)/(%d+)')
	return format(FULLDATE, Recent:GetFullDate(tonumber(weekday)+1, tonumber(m), tonumber(d), 2000+tonumber(y)))
end

function Recent:formattedTime(...)
	local hour = tonumber(date("%H", ...))
	local minutes = tonumber(date("%M", ...))
	return (GameTime_GetFormattedTime(hour,minutes,true))
end

function Recent:sortDate(...)
	local m,d,y = string.match(date("%x",...), '(%d+)/(%d+)/(%d+)')
	return string.format('%s/%s/%s', y,m,d)
 end

function Recent:printtable()
    print('...................'.. table.getn(RecentDB))
    for v, p in pairs(RecentDB) do
       print(v, p.lnk, p.type, p.iid, p.sd )
    end
end

function Recent:openMount(link)
	if not InCombatLockdown() then
		local mountID = 0
		local _, _, _, linkType, linkID = strsplit(":|H", link)
		if linkType == "item" then
			mountID = C_MountJournal.GetMountFromItem(tonumber(linkID))
		elseif linkType == "spell" then
			mountID = C_MountJournal.GetMountFromSpell(tonumber(linkID))
		end
		if not CollectionsJournal then
			CollectionsJournal_LoadUI()
		end
		SetCollectionsJournalShown(true, COLLECTIONS_JOURNAL_TAB_INDEX_MOUNTS)
		MountJournal_SelectByMountID(mountID)
	end
end

function Recent:openPet(link)
   	if not InCombatLockdown() then
		local numPets, numOwned = C_PetJournal.GetNumPets()
		local out, speciesPOO
		local _, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",link)

		if linkType=="item" then
			local a, b, c, creatureID, d, e, f, g, h, j,k, displayID, speciesID = C_PetJournal.GetPetInfoByItemID(tonumber(linkID))
			   speciesPOO=speciesID
		 else
			local speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType = C_PetJournal.GetPetInfoByPetID(battlePetID); 
			speciesPOO=speciesID
		 end
	
		for i=1, numPets do
			local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name, icon, petType, _, _, _, _, canBattle = C_PetJournal.GetPetInfoByIndex(i)

			if (speciesID == speciesPOO) then 
				out = petID 
				petIndex = i
				break;
			end
		end

		if petIndex then 
			if not CollectionsJournal then
				CollectionsJournal_LoadUI()
			end
			SetCollectionsJournalShown(true, COLLECTIONS_JOURNAL_TAB_INDEX_PETS)
			PetJournalPetList_UpdateScrollPos(PetJournal.listScroll, petIndex)
			PetJournal_ShowPetCardByID(out)
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFF5555Recent:|r Pet is still to learn!", 1.0, 0.8, 0.2);
		end
	end
end

function Recent:setNew(vID)
	local indexOffset = (WardrobeCollectionFrame.ItemsCollectionFrame.PagingFrame:GetCurrentPage() - 1) * WardrobeCollectionFrame.ItemsCollectionFrame.PAGE_SIZE
	for i = 1, WardrobeCollectionFrame.ItemsCollectionFrame.PAGE_SIZE do
	   local model = WardrobeCollectionFrame.ItemsCollectionFrame.Models[i]
	   local index = i + indexOffset;
	   local visualInfo = WardrobeCollectionFrame.ItemsCollectionFrame.filteredVisualsList[index]
	   if visualInfo  then
		  if visualInfo.visualID==vID then 
			 model.NewString:Show()
			 model.NewGlow:Show()
			 break
		  end
	   end
	end
end

function Recent_UpdateTooltip(self)
	if (self:GetParent().index) then
		local bpet=false
		GameTooltip:SetOwner(self, "ANCHOR_LEFT")
		if (RDB[self:GetParent().index].type=="m") then
			local _, visualID, _, icon, _, itemLink, link = C_TransmogCollection.GetAppearanceSourceInfo(RDB[self:GetParent().index].iid)
			GameTooltip:SetHyperlink(itemLink)

		elseif (RDB[self:GetParent().index].type=="p") then
			local _, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",RDB[self:GetParent().index].lnk)
			if linkType == "battlepet" then		
				local _, speciesID, level, breedQuality, maxHealth, power, speed, battlePetID = strsplit(":", RDB[self:GetParent().index].lnk)
				BattlePetToolTip_Show(tonumber(speciesID), tonumber(level), tonumber(breedQuality), tonumber(maxHealth), tonumber(power), tonumber(speed), string.gsub(string.gsub(RDB[self:GetParent().index].lnk, "^(.*)%[", ""), "%](.*)$", ""))
				bpet=true
			else
				GameTooltip:SetHyperlink(RDB[self:GetParent().index].lnk)
			end
		else
			GameTooltip:SetHyperlink(RDB[self:GetParent().index].lnk)
		end

		local ltext=""
		local rtext=""

		if (RDB[self:GetParent().index].type=="n" or RDB[self:GetParent().index].type=="m" or RDB[self:GetParent().index].type=="p") then
			rtext = L["<Right Click> for preview"]
		end

		if (RDB[self:GetParent().index].type=="a" or RDB[self:GetParent().index].type=="t" or RDB[self:GetParent().index].type=="n" or RDB[self:GetParent().index].type=="m" or RDB[self:GetParent().index].type=="p") then
			ltext = L["<Left Click> for collection"]
		end

		if (RDB[self:GetParent().index].type=="i") then
			itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(RDB[self:GetParent().index].lnk)
			if (itemEquipLoc == "INVTYPE_HEAD" or itemEquipLoc=="INVTYPE_SHOULDER" or itemEquipLoc=="INVTYPE_BODY" or itemEquipLoc=="INVTYPE_CHEST" or itemEquipLoc=="INVTYPE_ROBE" or itemEquipLoc=="INVTYPE_WAIST"  or itemEquipLoc=="INVTYPE_LEGS"  or itemEquipLoc=="INVTYPE_FEET"  or itemEquipLoc=="INVTYPE_WRIST"  or itemEquipLoc=="INVTYPE_HAND"  or itemEquipLoc=="INVTYPE_CLOAK"  or itemEquipLoc=="INVTYPE_WEAPON"  or itemEquipLoc=="INVTYPE_SHIELD"  or itemEquipLoc=="INVTYPE_2HWEAPON"  or itemEquipLoc=="INVTYPE_WEAPONMAINHAND"  or itemEquipLoc=="INVTYPE_WEAPONOFFHAND"  or itemEquipLoc=="INVTYPE_HOLDABLE" or itemEquipLoc=="INVTYPE_RANGED" or itemEquipLoc=="INVTYPE_THROWN" or itemEquipLoc=="INVTYPE_RANGEDRIGHT" or itemEquipLoc=="INVTYPE_TABARD") then
				rtext = L["<Right Click> for preview"]
			end	
		end

		if (rtext ~= "" or ltext ~= "" ) then
			if bpet then
				BattlePetTooltip:AddLine("_______________________________",0.5,0.5,0.5)
				BattlePetTooltip:AddLine(" ",1,1,1)
				BattlePetTooltip:AddLine(ltext,1,1,1)
				BattlePetTooltip:AddLine(rtext,1,1,1)
			else
				GameTooltip:AddLine(" ",1,1,1)
				GameTooltip:AddDoubleLine(ltext ,rtext,1,1,1,1,1,1)
				GameTooltip:AddLine(" ",1,1,1)
				GameTooltip:Show()
			end
		end
	end
end
   
function RecentUpdateList()

	local scrollFrame = RecentListFrame.ListScrollFrame
	local offset, _ = HybridScrollFrame_GetOffset(scrollFrame)
	local buttons = scrollFrame.buttons
	local prevdate=Recent:sortDate(time())
	local numDisplayedObjects = table.getn(RDB)
	local missinginfos=0

	for i=1, #buttons-1 do
		local itemName="x"
		local texture, itemName, itemClassID, itemSubClassID,itemRarity, itemLevel
		local button = buttons[i]
		local displayIndex = numDisplayedObjects - (i + offset)+1;

		if ( displayIndex > 0 ) then       
			local index = displayIndex
			local meta = nil
			button.factionIcon:SetShown(false)
			if index> 0 and (RDB[index].type == 'h') and index <= numDisplayedObjects then 

				button.name:SetFontObject("GameFontNormalLarge")
				button.name:SetJustifyH("CENTER")
				button.name:SetText('» ' .. Recent:localizedDate(RDB[index].ts) .. ' «')
				button.level:SetText(nil)
				button.icon:SetAlpha(0)
				prevdate=RDB[index].sd 
				button.selected = false
				button.selectedTexture:Hide()
				button.background:SetVertexColor(0, 0, 0, 0)
				button:SetHighlightTexture(nil)
				button.iconBorder:Hide()
				button.index = nil
				button.DragButton:SetEnabled(false)
				button.DragButton.ActiveTexture:Hide()
				button.name:SetTextColor(1,1,1,1)
				button.new:SetShown(false)
				button.new:SetText("")
				button.newGlow:SetShown(false)
				button:Show()

			elseif index <= numDisplayedObjects and index>0 then

				if (RDB[index].type == "a" ) then			-- Achievement
					id, itemName, points, completed, month, day, year, description, flags, texture, rewardText, isGuild, wasEarnedByMe, earnedBy = GetAchievementInfo(RDB[index].iid)							
					itemClassID = nil
					itemSubClassID = nil
					itemRarity = 2
					button.name:SetTextColor(1,1,1,1)
					button.level:SetText("")
					if completed== false and RDB[index].addinfo then 
						itemName=RDB[index].addinfo 
						meta=true
					end

				elseif (RDB[index].type == "m") then									--mog
					local _, visualID, _, icon, _, itemLink, link = C_TransmogCollection.GetAppearanceSourceInfo(RDB[index].iid)
					itemName, _, _, _, _, _,_, _, _, _,	_, _, _, _, _,_, _ = GetItemInfo(itemLink)			
					texture=icon
					itemClassID = nil
					itemSubClassID = nil
					button.name:SetTextColor(1,0.5,1,1)
					button.level:SetText("")
					if (itemName==nil) then
						itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(RDB[index].lnk) 
						itemRarity = 1
						itemLevel = 0
						_ , itemName = RDB[index].lnk:match("|H(.*)|h%[(.*)%]|h")
					end

				elseif (RDB[index].type == "i" or RDB[index].type == "t" or RDB[index].type == "p" or RDB[index].type == "n") then			--Item
					
					local _, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",RDB[index].lnk)

					if linkType=="battlepet" then

						local link = C_PetJournal.GetBattlePetLink(battlePetID)
						local _, _, _, _, rarity = C_PetJournal.GetPetStats(battlePetID);

						if rarity then itemRarity=rarity - 1 end
						local speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType = C_PetJournal.GetPetInfoByPetID(battlePetID);
						itemLevel=level
						texture=petIcon
						itemName=petName

						if (itemRarity==nil) then
							missinginfos = 1
							itemRarity = 1
							itemLevel = 0
							_ , itemName = RDB[index].lnk:match("|H(.*)|h%[(.*)%]|h")
						end

					else
						itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture,
						itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(RDB[index].lnk)

						if (itemRarity==nil) then
							itemID, itemType, itemSubType, itemEquipLoc, texture, itemClassID, itemSubClassID = GetItemInfoInstant(RDB[index].lnk) 
							itemRarity = 1
							itemLevel = 0
							_ , itemName = RDB[index].lnk:match("|H(.*)|h%[(.*)%]|h")
							missinginfos = 1
						else
							itemLevel, isPreview, baseILvl = GetDetailedItemLevelInfo(RDB[index].lnk)
						end
					end

					local red, green, blue, hex=GetItemQualityColor(itemRarity)	
					button.name:SetTextColor(red,green,blue,1)
					button.level:SetTextColor(red,green, blue,1)
					button.level:SetText("[".. itemLevel .. "]")

				end
				button.factionIcon:SetVertexColor(1, 1, 1, 1) 

				if (RDB[index].type == "p") then
					button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\pet")
					button.factionIcon:SetShown(true)

				elseif (RDB[index].type == "n") then
					button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\mount")
					button.factionIcon:SetShown(true)

				elseif (RDB[index].type == "t" ) then
					button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\toy")
					button.factionIcon:SetShown(true)

				elseif (RDB[index].type == "a" ) then
					button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\ACHIEVEMENT")
					button.factionIcon:SetShown(true)
					if meta==true then 
						button.factionIcon:SetVertexColor(0.3, 0.3, 0.8, 1) 
					end

				elseif (RDB[index].type == "m" ) then
					button.factionIcon:SetTexture("Interface\\AddOns\\Recent\\media\\mog")
					button.factionIcon:SetShown(true)

				else
				button.factionIcon:SetTexture("")
					button.factionIcon:SetShown(false)
				end
				
				local count=""
				if RDB[index].qnty ~= nil then
					if tonumber(RDB[index].qnty) >= 2 then count= "|cFFAAAAAA x "..RDB[index].qnty.. "|r"  end
				end
				
				local nameText= itemName..count
				button.level:SetJustifyH("RIGHT")
				button.name:SetJustifyH("LEFT")
				button.name:SetText("   ".. nameText)
				button.icon:SetTexture(texture)
				button:SetHighlightTexture("Interface\\AddOns\\Recent\\media\\whitedot", "ADD") 
				button.index = index
				button.spellID = index

				button.active = active
				button:Show()
				
				if ( RecentListFrame.selectedSpellID == index ) then 
					button.selected = true;
					button.selectedTexture:Show()
					button.selectedTexture:SetTexture("Interface\\AddOns\\Recent\\media\\yellowdot", "ADD")	
					button.DragButton.ActiveTexture:Show()
				else
					button.selected = false
					button.selectedTexture:Hide()
					button.DragButton.ActiveTexture:Hide()
				end
				
				button:SetEnabled(true);
				button.background:SetVertexColor(1, 1, 1, 1)
				button.DragButton:SetEnabled(true)
				button.additionalText = nil
				button.icon:SetDesaturated(false)
				button.icon:SetAlpha(1.0)
				button.name:SetFontObject("GameFontNormal")

				if ( button.showingTooltip ) then
					MountJournalMountButton_UpdateTooltip(button)
				end
			end
		else
			button:Hide()
		end
	 end

	 if (missinginfos==1) then 	C_Timer.After(1, function() RecentUpdateList() end)  end
	 local totalHeight = (table.getn(RDB)) * 46
	 HybridScrollFrame_Update(RecentListFrame.ListScrollFrame, totalHeight, RecentListFrame.ListScrollFrame:GetHeight())
end

function Recent:RecentSetSelection(number)
	if (number and table.getn(RDB)>0) then
		RecentListFrame.selectedSpellID=number
		if (RDB[number].loc) then 
			RecentListFrame.BottomLeftInset.SlotRequirementLabel:Show()
			RecentListFrame.BottomLeftInset.WhenLabel:Show()
			RecentListFrame.BottomLeftInset.WhereLabel:Show()
			RecentListFrame.BottomLeftInset.SlotLabel:Show()
			RecentListFrame.BottomLeftInset.WhatLabel:Show()
			RecentListFrame.BottomLeftInset.WhatText:Show()
			RecentListFrame.BottomLeftInset.SlotLabel:SetText(Recent:formattedTime(RDB[number].ts) .. L[" "] .. " - " .. Recent:localizedDate(RDB[number].ts))
			RecentListFrame.BottomLeftInset.SlotRequirementLabel:SetText(RDB[number].loc) 

			if (RDB[number].type=="i" or RDB[number].type=="t" or RDB[number].type=="n" or RDB[number].type=="p") then
				local itemName, _itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture,
				itemSellPrice, itemClassID, itemSubClassID, bindType, expacID, itemSetID, isCraftingReagent = GetItemInfo(RDB[number].lnk)		
				if itemRarity==nil then
					local _, _, _, linkType, linkID, _, _, _, _, _, battlePetID = strsplit(":|H",RDB[number].lnk)
					local speciesID, customName, level, xp, maxXp, displayID, isFavorite, petName, petIcon, petType = C_PetJournal.GetPetInfoByPetID(battlePetID); 
					if petType then
						itemEquipLoc="" 
						itemType=_G["TOOLTIP_BATTLE_PET"]	
						itemSubType=_G["BATTLE_PET_NAME_"..petType]
					end
				end
				RecentListFrame.BottomLeftInset.WhatText:SetText(itemType .. " > " .. (string.len(itemEquipLoc)>0 and itemSubType .. " > ".. _G[itemEquipLoc] or itemSubType))  

			elseif (RDB[number].type=="m") then
				RecentListFrame.BottomLeftInset.WhatText:SetText(L["Transmogification"])
				
			elseif (RDB[number].type=="a") then
				RecentListFrame.BottomLeftInset.WhatText:SetText(L["Achievement"])
			end

		else
			RecentListFrame.BottomLeftInset.SlotRequirementLabel:Hide()
			RecentListFrame.BottomLeftInset.SlotLabel:Hide()
			RecentListFrame.BottomLeftInset.WhenLabel:Hide()
			RecentListFrame.BottomLeftInset.WhereLabel:Hide()
			RecentListFrame.BottomLeftInset.WhatLabel:Hide()
			RecentListFrame.BottomLeftInset.WhatText:Hide()
		end
		
		RecentListFrame.BottomLeftInset.WhenLabel:SetText(L[" When:"])
		RecentListFrame.BottomLeftInset.WhereLabel:SetText(L[" Where:"])
		RecentListFrame.BottomLeftInset.WhatLabel:SetText(L[" What:"])
		RecentUpdateList()
	end
end

function RecentListItem_OnClick(self, button)
	Recent:RecentSetSelection(self.index)
end

function RecentJunk_OnClick()
	local isChecked = RecentListFrame.RecentJunkButton:GetChecked()
	if isChecked then
		RDB=RecentDB
	else
		RDB=RecentFiltered
	end
	RecentUpdateList()
end

function RecentListDragItem_OnClick(self, button)
	if not InCombatLockdown() then
		if button == "LeftButton" then
			if not CollectionsJournal then
				_G.CollectionsJournal_LoadUI()
			end
		
			if RDB[self:GetParent().index].type == "p" then
				Recent:openPet(RDB[self:GetParent().index].lnk)

			elseif RDB[self:GetParent().index].type == "n" then
					Recent:openMount(RDB[self:GetParent().index].lnk)
		
			elseif RDB[self:GetParent().index].type == "t" then
				if (RDB[self:GetParent().index].iid ~= nil)  then

					SetCollectionsJournalShown(true, COLLECTIONS_JOURNAL_TAB_INDEX_TOYS)		
					local page = ToyBox_FindPageForToyID(RDB[self:GetParent().index].iid)
					if page then
						ToyBox.PagingFrame:SetCurrentPage(page)
					end
				end
				
			elseif RDB[self:GetParent().index].type == "m" then
		
					local categoryID, visualID = C_TransmogCollection.GetAppearanceSourceInfo(RDB[self:GetParent().index].iid)
					local _, color, _, linkType, linkID = strsplit(":|H", RDB[self:GetParent().index].lnk)
					local mylink=linkType .. ":" .. tostring(linkID)                
					WardrobeCollectionFrame_OpenTransmogLink(mylink, nil)
					WardrobeCollectionFrame.ItemsCollectionFrame:RefreshVisualsList()
					WardrobeCollectionFrame.ItemsCollectionFrame:SelectVisual(visualID)
					WardrobeCollectionFrame.ItemsCollectionFrame:SetChosenVisualSource(visualID, RDB[self:GetParent().index].iid)
					C_Timer.After(0.25, function() Recent:setNew(visualID) end)  
		
			elseif RDB[self:GetParent().index].type == "a" then
		
					if not AchievementFrame then
						AchievementFrame_LoadUI()
					end
					ShowUIPanel(AchievementFrame)
					AchievementFrame_SelectAchievement(RDB[self:GetParent().index].iid)
			end
		
		elseif button == "RightButton" then

			if RDB[self:GetParent().index].type == "m" then
					local _, color, _, linkType, linkID = strsplit(":|H", RDB[self:GetParent().index].lnk);
					categoryID, visualID, canEnchant, icon, _, itemLink, transmogLink, _ = C_TransmogCollection.GetAppearanceSourceInfo(linkID)
					DressUpItemLink(itemLink)

			elseif RDB[self:GetParent().index].type == "i" or RDB[self:GetParent().index].type == "p" or RDB[self:GetParent().index].type == "n" then
					DressUpBattlePetLink(RDB[self:GetParent().index].lnk)
					DressUpMountLink(RDB[self:GetParent().index].lnk)
					DressUpItemLink(RDB[self:GetParent().index].lnk)	
			end
		end
		
		Recent:RecentSetSelection(self:GetParent().index)
	end
end

function Recent:OnChatCommand(msg)
	if msg and (msg == 'clear') then
		RecentDB = {}
		RDB={}
		RecentFiltered={}
		RecentUpdateList()

		RecentListFrame.BottomLeftInset.SlotRequirementLabel:Hide()
		RecentListFrame.BottomLeftInset.SlotLabel:Hide()
		RecentListFrame.BottomLeftInset.WhenLabel:Hide()
		RecentListFrame.BottomLeftInset.WhereLabel:Hide()
		RecentListFrame.BottomLeftInset.WhatLabel:Hide()
		RecentListFrame.BottomLeftInset.WhatText:Hide()

		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF5555Recent:|r List has been cleared...", 1.0, 0.8, 0.2);

	elseif msg and (msg == 'list') then 
		Recent:printtable()

	elseif msg and (msg=='togglemm') then	
		self.db.profile.minimap.hide = not self.db.profile.minimap.hide
		if self.db.profile.minimap.hide then
			icon:Hide("Recent")
		else
			icon:Show("Recent")
		end

	elseif msg and (msg == 'help') then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF5555Recent|r - your very personal log", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("by |cFFEE4444Finnegan/Antonidas(EU)|r", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("---------------------------", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("/togglemm - hide minimap button", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("/clear - delete entire history", 1.0, 0.8, 0.2);
		DEFAULT_CHAT_FRAME:AddMessage("/help - show this again", 1.0, 0.8, 0.2);
	else
	   	RecentListFrame:Show()
	end
end

tinsert(UISpecialFrames, "RecentListFrame");
UIPanelWindows["RecentListFrame"] = nil;
